Class: Gridium::Config
- Inherits:
-
Object
- Object
- Gridium::Config
- Defined in:
- lib/gridium.rb
Instance Attribute Summary collapse
-
#browser ⇒ Object
Returns the value of attribute browser.
-
#browser_source ⇒ Object
Returns the value of attribute browser_source.
-
#element_timeout ⇒ Object
Returns the value of attribute element_timeout.
-
#highlight_duration ⇒ Object
Returns the value of attribute highlight_duration.
-
#highlight_verifications ⇒ Object
Returns the value of attribute highlight_verifications.
-
#log_level ⇒ Object
Returns the value of attribute log_level.
-
#page_load_timeout ⇒ Object
Returns the value of attribute page_load_timeout.
-
#project_name_for_s3 ⇒ Object
Returns the value of attribute project_name_for_s3.
-
#report_dir ⇒ Object
Returns the value of attribute report_dir.
-
#screenshot_on_failure ⇒ Object
Returns the value of attribute screenshot_on_failure.
-
#screenshots_to_s3 ⇒ Object
Returns the value of attribute screenshots_to_s3.
-
#subdirectory_name_for_s3 ⇒ Object
Returns the value of attribute subdirectory_name_for_s3.
-
#target_environment ⇒ Object
Returns the value of attribute target_environment.
-
#url ⇒ Object
Returns the value of attribute url.
-
#visible_elements_only ⇒ Object
Returns the value of attribute visible_elements_only.
Instance Method Summary collapse
-
#initialize ⇒ Config
constructor
A new instance of Config.
Constructor Details
#initialize ⇒ Config
Returns a new instance of Config.
26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 |
# File 'lib/gridium.rb', line 26 def initialize @report_dir = Dir.home @browser_source = :local #if browser source is set to remote, target environment needs to be set properly @target_environment = "localhost" @browser = :firefox @url = "about:blank" @page_load_timeout = 15 @element_timeout = 15 #This needs to be changed to only look for an element after a page is done loading @visible_elements_only = true @log_level = :fatal @highlight_verifications = false @highlight_duration = 0.100 @screenshot_on_failure = false @screenshots_to_s3 = false @project_name_for_s3 = 'gridium' @subdirectory_name_for_s3 = '' #rely on GridiumS3 default end |
Instance Attribute Details
#browser ⇒ Object
Returns the value of attribute browser.
23 24 25 |
# File 'lib/gridium.rb', line 23 def browser @browser end |
#browser_source ⇒ Object
Returns the value of attribute browser_source.
23 24 25 |
# File 'lib/gridium.rb', line 23 def browser_source @browser_source end |
#element_timeout ⇒ Object
Returns the value of attribute element_timeout.
23 24 25 |
# File 'lib/gridium.rb', line 23 def element_timeout @element_timeout end |
#highlight_duration ⇒ Object
Returns the value of attribute highlight_duration.
24 25 26 |
# File 'lib/gridium.rb', line 24 def highlight_duration @highlight_duration end |
#highlight_verifications ⇒ Object
Returns the value of attribute highlight_verifications.
24 25 26 |
# File 'lib/gridium.rb', line 24 def highlight_verifications @highlight_verifications end |
#log_level ⇒ Object
Returns the value of attribute log_level.
23 24 25 |
# File 'lib/gridium.rb', line 23 def log_level @log_level end |
#page_load_timeout ⇒ Object
Returns the value of attribute page_load_timeout.
23 24 25 |
# File 'lib/gridium.rb', line 23 def page_load_timeout @page_load_timeout end |
#project_name_for_s3 ⇒ Object
Returns the value of attribute project_name_for_s3.
24 25 26 |
# File 'lib/gridium.rb', line 24 def project_name_for_s3 @project_name_for_s3 end |
#report_dir ⇒ Object
Returns the value of attribute report_dir.
23 24 25 |
# File 'lib/gridium.rb', line 23 def report_dir @report_dir end |
#screenshot_on_failure ⇒ Object
Returns the value of attribute screenshot_on_failure.
24 25 26 |
# File 'lib/gridium.rb', line 24 def screenshot_on_failure @screenshot_on_failure end |
#screenshots_to_s3 ⇒ Object
Returns the value of attribute screenshots_to_s3.
24 25 26 |
# File 'lib/gridium.rb', line 24 def screenshots_to_s3 @screenshots_to_s3 end |
#subdirectory_name_for_s3 ⇒ Object
Returns the value of attribute subdirectory_name_for_s3.
24 25 26 |
# File 'lib/gridium.rb', line 24 def subdirectory_name_for_s3 @subdirectory_name_for_s3 end |
#target_environment ⇒ Object
Returns the value of attribute target_environment.
23 24 25 |
# File 'lib/gridium.rb', line 23 def target_environment @target_environment end |
#url ⇒ Object
Returns the value of attribute url.
23 24 25 |
# File 'lib/gridium.rb', line 23 def url @url end |
#visible_elements_only ⇒ Object
Returns the value of attribute visible_elements_only.
23 24 25 |
# File 'lib/gridium.rb', line 23 def visible_elements_only @visible_elements_only end |